@@ -29,16 +29,30 @@ all the user actions to CEM fo logging/mentor input/game input."
 
 #include "../gui_clutter/instrument_testing.h"
 static int chum_count_themes();
-static void chum_play_recorded_and_backing_track();
+
 
 /*Chum is like state-machine*/
 int state_of_chum=SCREEN_WELCOME;
 int current_theme; /*for composing game*/
-int current_song;  /*for karaoke game*/
+char* current_song;  /*for karaoke game*/
 
 //This is not same than language from profile (which affects speech of mentor)
 //This is only when user wants sing a song in different language.
-char* selected_language="";
+char* selected_language="_fi";
+
+char* chum_get_selected_language(){
+	return selected_language;
+}
+
+void  chum_set_selected_language(gchar* language){
+	selected_language=language;
+}
+
+
+gboolean duetto=TRUE;
+void chum_set_duetto(gboolean b){
+	duetto=b;
+}
 
 //This helps logging/debugging state switching
 char *name_of_states[] ={ "SCREEN_WELCOME",
@@ -54,12 +68,14 @@ char *name_of_states[] ={ "SCREEN_WELCOME",
 
 static JammoSequencer* sequencer = NULL;
 JammoEditingTrack* editing_track;
+JammoEditingTrack* editing_track_partner; //pairgame for youngest group
 JammoEditingTrack* backing_track;
 
 JammoInstrumentTrack* instrument_track;
 JammoSliderTrack* slider_track;
 JammoRecordingTrack* recording_track;
 
+gboolean multiplaying=FALSE; 
 gint number_of_themes = 0;
 gint chum_get_number_of_themes(){
 return number_of_themes;
@@ -78,6 +94,8 @@ void chum_start_jammo_main_loop() {
 
 void chum_start_chum(int argc, char** argv){
     int i;
+    int port;
+    char strHostName[100];
     #ifdef N900
     gchar *cmd = g_strdup_printf("mkdir -p /home/user/.jammo");
     int error_code = system(cmd);
@@ -110,6 +128,19 @@ void chum_start_chum(int argc, char** argv){
     if (argc==2 && strcmp(argv[1],"password")==0) { 
         state_of_chum=SCREEN_PASSWORD_TYPING;
         }
+    else if (argc==3 && strcmp(argv[1],"--server")==0) { //Test communication in server mode
+		printf("\nCreating server");
+        	port=atoi(argv[2]);
+		gems_create_game_server(port);
+		multiplaying=TRUE;
+        }
+     else if (argc==4 && strcmp(argv[1],"--client")==0) { //Test communication in client mode
+		printf("\nCreating client");		
+		strcpy(strHostName,argv[2]);
+		port=atoi(argv[3]);
+        	gems_create_game_client(strHostName, port);
+		multiplaying=TRUE;
+        }
 
     gui_init();
     if (argc==2 && strcmp(argv[1],"sing")==0) { 
@@ -274,13 +305,14 @@ void chum_change_theme(int theme_number){
 	}
 	sequencer = jammo_sequencer_new();
 	editing_track = chum_create_new_empty_track(); //for samples
-	backing_track = chum_create_new_empty_track(); //for backing track
-
+	if (multiplaying)
+		editing_track_partner = chum_create_new_empty_track(); //for partners samples
 
     //Add backing track if there are.
     gchar* filepath1 = g_strdup_printf("%s/%s/backing_track.wav", THEMES_DIR, chum_get_theme_folder(chum_get_current_theme()));
     int fd = open(filepath1,O_RDONLY);
     if (fd != -1) {
+	backing_track = chum_create_new_empty_track(); //for backing track
 	sample = jammo_sample_new_from_file(filepath1);
 	jammo_editing_track_add_sample(backing_track, sample, 0);
 	close(fd);
@@ -296,8 +328,8 @@ chum_change_theme(theme_number);
     gui_go_screen(state_of_chum);
 }
 
-int chum_get_current_song(){
-cem_add_to_log(g_strdup_printf("get_current_song(). will return %d",current_song),LOG_DEBUG);
+char* chum_get_current_song(){
+cem_add_to_log(g_strdup_printf("get_current_song(). will return '%s'",current_song),LOG_DEBUG);
     return current_song;
 }
 
@@ -313,10 +345,10 @@ return name;
 #define RECORDING_FILENAME "/tmp/recording.wav"
 #endif
 
-/*STATIC
+/*
 Always call this before gui_go_screen(SCREEN_PLAY_RECORDED);
 */
-static void chum_play_recorded_and_backing_track(){
+void chum_play_recorded_and_backing_track(){
     chum_stop_sequencer();
     /*
 	if (sequencer) {
@@ -326,32 +358,61 @@ static void chum_play_recorded_and_backing_track(){
 	sequencer = jammo_sequencer_new();
 
 	backing_track = chum_create_new_empty_track(); //for comping track
-    const gchar *song_folder=chum_get_song_folder(chum_get_current_song());
     //gchar *filename = g_strdup_printf("%s/comping.ogg", song_folder);
     //printf("language is '%s'\n",selected_language);
-    gchar *filename = g_strdup_printf("%s/comping%s.ogg", song_folder,selected_language);
-    chum_add_new_sample_to_track_nth_slot(backing_track,filename,0);
+
+    gchar *filename = g_strdup_printf("%s/songs/%s/comping%s.ogg", DATA_DIR,chum_get_current_song(),selected_language);
+    int fd = open(filename,O_RDONLY);
+    if (fd == -1) //File not found
+     {
+     filename = g_strdup_printf("%s/songs/%s/comping.ogg", DATA_DIR,chum_get_current_song()); //use without language postfix
+     }
+
+    chum_add_new_sample_to_track_nth_slot(backing_track, NULL,filename,0);
+    g_free(filename);
 
     editing_track = chum_create_new_empty_track(); //for recorded singing
-    chum_add_new_sample_to_track_nth_slot(editing_track,RECORDING_FILENAME,0);
-
+    chum_add_new_sample_to_track_nth_slot(editing_track,NULL,RECORDING_FILENAME,0);
+    state_of_chum=SCREEN_PLAY_RECORDED;
 }
 
 
-void chum_change_to_song(int song_number){
+void chum_change_to_song(const char *song_name){
   JammoSample* sample1;
 
-    cem_add_to_log(g_strdup_printf("chum_change_to_song(%d)",song_number),LOG_DEBUG);
-    current_song=song_number;
+    cem_add_to_log(g_strdup_printf("chum_change_to_song(%s)",song_name),LOG_DEBUG);
+    current_song=song_name;
 
   if (sequencer) {
     g_object_unref(sequencer);
   }
 
-  gchar *filename = g_strdup_printf("%s/comping%s.ogg", chum_get_song_folder(song_number),selected_language);
-
+  //printf("will make sample for file '%s'\n",filename);
   sequencer = jammo_sequencer_new();
 
+   gchar *filename;
+   int fd;
+   if (duetto)
+     {
+     filename = g_strdup_printf("%s/songs/%s/vocal%s.ogg", DATA_DIR,song_name,selected_language);
+     fd = open(filename,O_RDONLY);
+
+     if (fd == -1) //File not found
+       {  //no vocal track. use only comping.
+       filename = g_strdup_printf("%s/songs/%s/comping.ogg", DATA_DIR,song_name); //try without language postfix
+       }
+     }
+   else //solo
+    {
+    filename = g_strdup_printf("%s/songs/%s/comping%s.ogg", DATA_DIR,song_name,selected_language);
+    fd = open(filename,O_RDONLY);
+       if (fd == -1) //File not found
+       {  //no track for this language,. use  common comping.
+       filename = g_strdup_printf("%s/songs/%s/comping.ogg", DATA_DIR,song_name); //try without language postfix
+       }
+    }
+
+  printf("looking for file '%s' \n",filename);
   backing_track = jammo_editing_track_new();   //for comping track
   jammo_sequencer_add_track(sequencer, JAMMO_TRACK(backing_track));
 
@@ -362,7 +423,8 @@ void chum_change_to_song(int song_number){
   jammo_sequencer_add_track(sequencer, JAMMO_TRACK(recording_track));
 
   state_of_chum=SCREEN_SONG_RECORDING;
-  gui_go_screen(state_of_chum);
+  g_free(filename);
+  //gui_go_screen(state_of_chum);
 }
 
 void chum_stop_sequencer(){
@@ -390,32 +452,72 @@ void chum_get_duration_of_song(guint64 *value) {
 //FIX: implement this.
 //void chum_pause_sequencer();
 
-JammoSample* chum_add_new_sample_to_track(JammoEditingTrack* track, const gchar* srcLocation, guint64 startTime){
+
+/*
+srcLocation = Full path.
+
+Reasons to call this:
+-local sample add			(pass this to gems)
+-local sample add to backing_track			(track==backing_track)
+-remote sample add			(track!=editing_track)
+-local singing game playback		(id==NULL)
+
+*/
+JammoSample* chum_add_new_sample_to_track(JammoEditingTrack* track,const gchar* id,  const gchar* srcLocation, guint64 startTime){
 	JammoSample* sample;
 
-    cem_add_to_log(g_strdup_printf("chum_add_new_sample_to_track(%p,%s,%" GST_TIME_FORMAT ")",track, srcLocation, GST_TIME_ARGS (startTime)),LOG_DEBUG);
-
+    cem_add_to_log(g_strdup_printf("chum_add_new_sample_to_track(%s,%s,%" GST_TIME_FORMAT ")", id,srcLocation, GST_TIME_ARGS (startTime)),LOG_DEBUG);
+	if (track==editing_track && id!=NULL)
+		{
+		//printf("pass to gems '%s'\n",id);
+	   	gems_add_new_sample_to_track(0, id, startTime);
+		}
+	else if (track==backing_track)
+		{
+		printf("Added sample to backing_track\n");
+		}
+	else 
+		{
+		printf("Partner added sample\n");
+		}
 	sample = jammo_sample_new_from_file(srcLocation);
 	jammo_editing_track_add_sample(track, sample, startTime);
 	return sample;
 }
 
 
-JammoSample* chum_add_new_sample_to_track_nth_slot(JammoEditingTrack* track, const gchar* srcLocation, guint slot){
-	JammoSample* sample;
-	
-    cem_add_to_log(g_strdup_printf("chum_add_new_sample_to_track_nth_slot(%p,%s,%d)",track, srcLocation, slot),LOG_DEBUG);
+/*
+Wrapper
+*/
+JammoSample* chum_add_new_sample_to_track_nth_slot(JammoEditingTrack* track, const gchar* id, const gchar* srcLocation, guint slot){
+return chum_add_new_sample_to_track(track, id, srcLocation, slot * GST_SECOND); /* TODO: Slot is one second now. */
+}
 
-	sample = jammo_sample_new_from_file(srcLocation);
-	jammo_editing_track_add_sample(track, sample, slot * GST_SECOND); /* TODO: Slot is one second now. */
-	return sample;
+
+JammoSample* chum_add_new_sample_to_track_remote(int a, const gchar* id, guint64 startTime){
+cem_add_to_log(g_strdup_printf(" chum_add_new_sample_to_track_remote(%d,%s,%" GST_TIME_FORMAT ")",a, id,GST_TIME_ARGS (startTime)),LOG_DEBUG);
+gchar* sound=object_add_to_timeline_remote(id,startTime);
+
+gchar* buffer0;
+buffer0 = g_strdup_printf("%s/themes/%s/%s",DATA_DIR,chum_get_theme_folder(chum_get_current_theme()), sound);
+return chum_add_new_sample_to_track(editing_track_partner,id,buffer0,startTime);
+}
+
+
+void chum_remove_sample_from_slot_remote(int a, int slot){
+	chum_remove_sample_from_slot(editing_track_partner, slot);
 }
 
 void chum_remove_sample_from_slot(JammoEditingTrack* track, int slot){
 	JammoSample* sample;
 
 cem_add_to_log(g_strdup_printf("chum_remove_sample_from_slot(track=%p,slot=%d)",track,slot),LOG_DEBUG);
-
+	if (track==editing_track)
+		{
+		gems_remove_sample_from_slot(1,slot);
+		}
+	else 
+		printf("Partner removed sample\n");
 	if ((sample = jammo_editing_track_get_sample(track, slot * GST_SECOND))) { /* TODO: Slot is one second now. */
 		jammo_editing_track_remove_sample(track, sample);
 	}
